/* FeatureIDE - A Framework for Feature-Oriented Software Development
 * Copyright (C) 2005-2019  FeatureIDE team, University of Magdeburg, Germany
 *
 * This file is part of FeatureIDE.
 *
 * FeatureIDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FeatureIDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with FeatureIDE.  If not, see <http://www.gnu.org/licenses/>.
 *
 * See http://featureide.cs.ovgu.de/ for further information.
 */
package de.ovgu.featureide.fm.core;

import java.util.regex.Pattern;

/**
 * Definition of valid operator names for constraint expressions.
 *
 * @author Fabian Benduhn
 */
public class Operator {

	public static final String[] NAMES = { "Not", "And", "Or", "Implies", "Iff", "(", ")" };

	public static final String REGEX;
	static {
		final StringBuilder sb = new StringBuilder();
		sb.append("(");
		for (final String keyword : NAMES) {
			if ((keyword == "(") || (keyword == ")")) {
				sb.append("\\" + keyword);
			} else {
				sb.append("\\b" + Pattern.quote(keyword.toLowerCase()) + "\\b");
			}
			sb.append("|");
		}
		sb.setCharAt(sb.length() - 1, ')');
		REGEX = sb.toString();
	}

	/**
	 * @param name
	 * @return true if name is operator name
	 */
	public static boolean isOperatorName(String name) {

		for (final String operator : NAMES) {
			if (name.toLowerCase().equals(operator.toLowerCase())) {
				return true;
			}
		}
		return false;
	}

}
