package icsoc20;

import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import com.opencsv.CSVReader;

/**
 * The main class and method for executing the ICSOC 2020 experiments
 * 
 * @author Andreas Metzger
 */
public class Main {

	
	// Environment data (will be read from file)
	public static double rewards[] = new double[344];
	public static String config[] = new String[344];
	// Specifics for the CloudRM exemplar: QoS values
	public static double energy[] = new double[344];
	public static double migrations[] = new double[344];
	
	// Mapping configurations to int = action (for Q-Learning) and vice versa
	public static Map<String,Integer> confAction = new HashMap<String,Integer>();
	public static Map<Integer,String> actionConf= new HashMap<Integer,String>();
	
	
	// --- Hyper-Parameters
	static double learningRate = .85;	// Std. Q-Learning Hyperparam.
	static double discountFactor = .2;	// Std. Q-Learning Hyperparam.
	
	static double epsilonDecayRate = 0.97;  // Std. E-Decay Hyperparam.
	static double deltaDecayRate = 0.90; 	// Decay Hyperparam. for FM-guided strategies
		
	// -- Experiment parameters
	static int nbrRuns = 100;		// repetitions of experiment
	static int maxEpisodes = 250; 	// number of episodes (i.e., time steps)
	static int avg = 10; 			// number of rewards to be averaged (to smooth the curves a little)

	static boolean evolution = true;  // True for running evolution experiments
	static int nbrEvolSteps = 4; 	  // Max number of evolution steps 
	
	static String resDir = "x-qlearning"; // Results directory

	// specifies which results to compute
	static int qln = 1;
	static int inc = 1;
	static int deg = 1;

	static int eqln = 1;
	static int einc = 1;
	static int edeg = 1;

	// ---	
	
	public static void main(String[] args) {

		try{
			
			// read rewards per config from file
			readRewards("./resources/cloudrm.csv");
	
			// output to CSV
			PrintWriter wr = new PrintWriter(new FileWriter("./dat/"+resDir+"/"+"res.csv"));	
			wr.println("e;q;qe;ie;ine;de;dne;Eq;Eqe;Eie;Eine;Ede;Edne;Mq;Mqe;Mie;Mine;Mde;Mdne;");
			
			// Epsilon Greedy - NON EVOL-AWARE
			System.out.println("\n========= e-Greedy - NO EVOL");
			EpsilonGreedyExploration ep = new EpsilonGreedyExploration(0.0, 344, false);
			double resultsQ[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double EresultsQ[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double MresultsQ[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			
			for(int k = 0; k < nbrRuns; k++) {
				System.out.print("\n### "+k+" ");
				if(qln == 1) learn(ep, resultsQ[k], EresultsQ[k], MresultsQ[k], evolution);				
			}

			// Epsilon Greedy - EVOL-AWARE
			System.out.println("\n========= e-Greedy - EVOL");
			EpsilonGreedyExploration eep = new EpsilonGreedyExploration(0.0, 344, true);
			double resultsQE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double EresultsQE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double MresultsQE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			
			for(int k = 0; k < nbrRuns; k++) {
				System.out.print("\n### "+k+" ");
				if(eqln == 1) learn(eep, resultsQE[k], EresultsQE[k], MresultsQE[k], evolution);				
			}

			
			// FM-Learning: Inc strategy - NON EVOL-AWARE
			System.out.println("\n\n========= FM-Learning: Incremental - NO EVOL");
			FMExploration epINE = new FMExploration(0.0, confAction, actionConf, 2, 344, false);
			double resultsINE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double EresultsINE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double MresultsINE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			
			for(int k = 0; k < nbrRuns; k++) {
				System.out.print("\n### "+k+" ");
				if(inc == 1) learn(epINE, resultsINE[k], EresultsINE[k], MresultsINE[k], evolution);
			}

			// FM-Learning: Inc strategy - EVOL-AWARE
			System.out.println("\n\n========= FM-Learning: Incremental - EVOL");
			FMExploration epIE = new FMExploration(0.0, confAction, actionConf, 2, 344, true);
			double resultsIE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double EresultsIE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double MresultsIE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];

			for(int k = 0; k < nbrRuns; k++) {
				System.out.print("\n### "+k+" ");
				if(einc == 1 ) learn(epIE, resultsIE[k], EresultsIE[k], MresultsIE[k], evolution);
			}

						
			// FM-Learning: Deg strategy - NON EVOL-AWARE
			System.out.println("\n\n========= FM-Learning: Feature Degree - NO EVOL");
			FMExploration epDNE = new FMExploration(0.0, confAction, actionConf, 3, 344, false);
			double resultsDNE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double EresultsDNE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double MresultsDNE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			
			for(int k = 0; k < nbrRuns; k++) {
				System.out.print("\n### "+k+" ");
				if(deg == 1) learn(epDNE, resultsDNE[k], EresultsDNE[k], MresultsDNE[k], evolution);
			}
						
			// FM-Learning: Deg strategy - EVOL-AWARE
			System.out.println("\n\n========= FM-Learning: Feature Degree - EVOL");
			FMExploration epDE = new FMExploration(0.0, confAction, actionConf, 3, 344, true);
			double resultsDE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double EresultsDE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];
			double MresultsDE[][] = new double[nbrRuns][maxEpisodes*nbrEvolSteps];

			for(int k = 0; k < nbrRuns; k++) {
				System.out.print("\n### "+k+" ");
				if(edeg == 1) learn(epDE, resultsDE[k], EresultsDE[k], MresultsDE[k], evolution);
			}
			
						
			// Generate results CSV (averaging the different runs)
			double rewQ = 0; double rewQE = 0; double rewIE = 0; double rewINE = 0; double rewDE = 0; double rewDNE = 0;
			double ErewQ = 0; double ErewQE = 0; double ErewIE = 0; double ErewINE = 0; double ErewDE = 0; double ErewDNE = 0;
			double MrewQ = 0; double MrewQE = 0; double MrewIE = 0; double MrewINE = 0; double MrewDE = 0; double MrewDNE = 0;
			
			// start CSV output after first complete avg reward was computed
			for(int i = avg; i < maxEpisodes*nbrEvolSteps; i++) {
				rewQ = 0;  rewQE = 0;  rewIE = 0;  rewINE = 0;  rewDE = 0;  rewDNE = 0;
				ErewQ = 0; ErewQE = 0; ErewIE = 0; ErewINE = 0; ErewDE = 0; ErewDNE = 0;
				MrewQ = 0; MrewQE = 0; MrewIE = 0; MrewINE = 0; MrewDE = 0; MrewDNE = 0;
				
				for(int k = 0; k < nbrRuns; k++) {
					rewQ +=    resultsQ[k][i]; 
					rewQE +=   resultsQE[k][i]; 
					rewIE +=   resultsIE[k][i];
					rewINE +=  resultsINE[k][i];
					rewDE +=   resultsDE[k][i];
					rewDNE +=  resultsDNE[k][i];
					
					ErewQ +=   EresultsQ[k][i];
					ErewQE +=  EresultsQE[k][i]; 
					ErewIE +=  EresultsIE[k][i];
					ErewINE += EresultsINE[k][i];
					ErewDE +=  EresultsDE[k][i];
					ErewDNE += EresultsDNE[k][i];

					MrewQ +=   MresultsQ[k][i];
					MrewQE +=  MresultsQE[k][i]; 
					MrewIE +=  MresultsIE[k][i];
					MrewINE += MresultsINE[k][i];
					MrewDE +=  MresultsDE[k][i];
					MrewDNE += MresultsDNE[k][i];

				}
				
				wr.println(""+i
				+";"+String.format(Locale.GERMANY, "%.10g", rewQ/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", rewQE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", rewIE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", rewINE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", rewDE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", rewDNE/(double)nbrRuns)
				
				+";"+String.format(Locale.GERMANY, "%.10g", ErewQ/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", ErewQE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", ErewIE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", ErewINE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", ErewDE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", ErewDNE/(double)nbrRuns)
				
				+";"+String.format(Locale.GERMANY, "%.10g", MrewQ/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", MrewQE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", MrewIE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", MrewINE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", MrewDE/(double)nbrRuns)
				+";"+String.format(Locale.GERMANY, "%.10g", MrewDNE/(double)nbrRuns)
				);

			}
			wr.close();

		} catch(Exception e) {
			e.printStackTrace();
		}
		
		System.out.println("\n\n Done");
		Toolkit.getDefaultToolkit().beep(); 

	}
	
	/**
	 * Implementation of the main learning loop, including the initialization of the Q-Table and
	 * iterations of get next state and update Q-Table
	 * 
	 * @param  ep 		 the exploration policy to use
	 * @param  results 	 an array of the rewards achieved
	 * @param  evolution consider evolution steps (or not)
	 */
	public static void learn(IExplorationPolicy ep, double results[], double Eresults[], double Mresults[], boolean evolution) throws Exception {
		
		// Init Rewards with -1.7, i.e., lowest reward possible (for CloudRM data set here)
		// If higher values, this means one implicitly defines exploration via optimistic initial values and
		// thus may overlay with the effect of e-greedy or FM-guided learning  
		QLearning ql = new QLearning(1, 344, ep, false, -1.7);
		
		// Hyper-parameters for Q-Learning
		ql.setDiscountFactor(discountFactor);
		ql.setLearningRate(learningRate);
		
		int action = 0;
		double reward = 0;

		// For smoothing the curves a little
		double avgreward[] = new double[avg];
		for(int k = 0; k < avg; k++) avgreward[k] = -1;
		double totalreward = 0;
		double avgE[] = new double[avg];
		for(int k = 0; k < avg; k++) avgE[k] = 0;
		double totalE = 0;
		double avgM[] = new double[avg];
		for(int k = 0; k < avg; k++) avgM[k] = -1;
		double totalM = 0;

		if(!evolution)
			nbrEvolSteps = 1;
		
		// Different evolution steps (if any)
		// The action numbers are manually determined from the CloudRM feature model
		for(int e = 0; e < nbrEvolSteps; e++) {
			switch(e) {
			case 0: 
				if(!evolution) {
					ql.setActions(344);
					ep.setMaxAction(344);
					ep.setPreviousMaxAction(0);
				} else {
				ql.setActions(26);
				ep.setMaxAction(26);
				ep.setPreviousMaxAction(0);
				}
				break;
			case 1:
				ql.setActions(56);
				ep.setMaxAction(56);
				ep.setPreviousMaxAction(26);
				break;				
			case 2:
				ql.setActions(128);
				ep.setMaxAction(128);
				ep.setPreviousMaxAction(56);
				break;		
			case 3:
				ql.setActions(344);
				ep.setMaxAction(344);
				ep.setPreviousMaxAction(128);
				break;		
			}
			
		
			double epsilon = 1.0;
			double delta = 1.0;
			
			for(int i = 0; i < maxEpisodes; i++) {
				
				if(i%50 == 0)
					System.out.print(".");
	
				// Implement epsilon decay
				ep.setEpsilon(epsilon);
				epsilon = epsilon * epsilonDecayRate;
	
				// Implement delta decay
				// (certain amount of randomness also for FM strategies at beginning)
				ep.setDelta(delta);
				delta = delta * deltaDecayRate;
				
				// Select action and get reward for that action
				action = ql.GetAction(0);
				reward = rewards[action];
				
				// compute rewards (smoothed via avg)
				avgreward[i % avg] = reward;
				totalreward = 0;
				for(int k = 0; k < avg; k++) totalreward += avgreward[k];		
				results[maxEpisodes*e + i] = totalreward/(double)avg;

				// compute QoS values
				avgE[i % avg] = energy[action];
				totalE = 0;
				for(int k = 0; k < avg; k++) totalE += avgE[k];		
				Eresults[maxEpisodes*e + i] = totalE/(double)avg;

				avgM[i % avg] = migrations[action];
				totalM = 0;
				for(int k = 0; k < avg; k++) totalM += avgM[k];		
				Mresults[maxEpisodes*e + i] = totalM/(double)avg;

				// Q-Learning
				ql.UpdateState(0, action, reward, 0);
			}
			
		}
	}

	/**
	 * Read the rewards per feature combination (configuration) from CSV file.
	 * NOTE: Configurations for the evolution aware experiments need to be sorted in such a way
	 * as to allow discerning the FM deltas according to the position in the file
	 * 
	 * @param  filename the filename of the CSV
	 */
	public static void readRewards(String filename) throws Exception {
		
			
			Reader myReader = new BufferedReader(new FileReader(new File(filename)));
			CSVReader myCSV = new CSVReader(myReader, ';');
					
			Iterator<String[]> entries = myCSV.iterator();
			String[] entry = null;
			entries.next(); // skip the header

			int k=0;
			Double tmp = 0.0;

			while(entries.hasNext()){
				entry = entries.next();
				tmp = Double.parseDouble(entry[0].replace(',','.'));
				rewards[k] = tmp;

				tmp = Double.parseDouble(entry[1].replace(',','.'));
				energy[k] = tmp;

				tmp = Double.parseDouble(entry[2].replace(',','.'));
				migrations[k] = tmp;
				
				config[k] = entry[3];
				
				confAction.put(config[k], k);
				actionConf.put(k, config[k]);
				
				k++;
			}
					
			myCSV.close();

		
	}
	
	

}
